/**************************************************************************
 * by AS - Bewsserungscomputer 
 * 
 * Version mit Dimmer
 *
 **************************************************************************/
#define F_CPU 12000000UL // Takt in Hertz
#define T_VAL 50000 // 12000000/50000=240 -> 239*50000+50350+adj // 50350 ermittelt, adj 0-99


#include <avr/io.h>
#include <util/delay.h> 
#include <string.h>
#include <avr/interrupt.h>


volatile uint8_t int_cnt=0;  // interrupt-zehler fr sekunden-takt

volatile uint8_t key_code=0; // code der abgefragten taste
volatile uint8_t key_long=0; // zahler fr langdruck

volatile uint8_t bl_soll=1; // Sollwert fr backlight 
volatile uint8_t bl_ist=0;  // Istwertwert fr backlight
volatile uint8_t bl_mode=1; // bl on / off
volatile uint16_t adj=50; // adjustment fr exakte Sekunde (alt=50430)


typedef struct
	{
		uint8_t sec;
		uint8_t min;
		uint8_t hour;
		uint8_t day;
		uint8_t wday;
		uint8_t month;
		uint8_t year;
		uint8_t dls_flag; // 1=Sommerzeit im Okt. bereits zurckgestellt
		uint8_t leapyear;
	} datestruct;
	
volatile datestruct zeit;
volatile datestruct szeit1_on, szeit1_off,szeit2_on, szeit2_off;
volatile datestruct szeit3_on, szeit3_off,szeit4_on, szeit4_off;
uint8_t kreis1,kreis2;


volatile uint8_t m_end[2][12] = {{31,28,31,30,31,30,31,31,30,31,30,31},
									{31,29,31,30,31,30,31,31,30,31,30,31}};

void calc_zeit(void) // Berechnet Uhrzeit, Datum, Sommerzeit etc.
{
	zeit.sec++;
	if (zeit.sec>59) { // Sekunde
		zeit.sec=0;
		zeit.min++;
		if (zeit.min>59) { // Minute
			zeit.min=0;
			zeit.hour++;
			//Sommerzeit im Mrz
			if (zeit.month==3 && zeit.wday==0 && zeit.day >= 25  &&zeit.hour==2) {
				zeit.hour++;
				zeit.dls_flag=0;
			}
			//Sommerzeit im Oktober Achtung: Durch Rckstellen der Uhr mgliche Endlosschleife daher Flag
			if (zeit.month==10 && zeit.wday==0 && zeit.day >= 25 && zeit.hour==3 && zeit.dls_flag==0) {
				zeit.hour--;
				zeit.dls_flag=1;
			}
			if(zeit.hour>23) { // Stunde
				zeit.hour=0;
				zeit.day++;
				zeit.wday++;
				if(zeit.wday>6) { // Wochentag
					zeit.wday=0;
				}
				//Schaltjahr
				if ((zeit.year%4==0)&&((zeit.year%100!=0)||(zeit.year%400==0)))
					zeit.leapyear = 1;
				else
					zeit.leapyear = 0;
				
				//Tag
				if (zeit.day>m_end[zeit.leapyear][zeit.month-1]) {
					zeit.day=1;
					zeit.month++;
				}
				//Jahr
				if (zeit.month>12) {
					zeit.month=1;
					zeit.year++;
				}
			}
		}
	}
}

void backlight(uint8_t mode) // mode schaltet Backlight 1=ein; 0=aus
{
	if(mode==1) {
		if(bl_ist==bl_soll) { // PWM fr Displayhellgkeit
			PORTD |= (1<<PD7); // bl on
			bl_ist=0;
		}
		else PORTD &= ~(1<<PD7); // bl 0ff
	}
	else PORTD &= ~(1<<PD7);
}

ISR(TIMER1_COMPA_vect) 
{
	// fortschreiben uhr und kalender
	int_cnt++;
	if (int_cnt>238) {
		OCR1A = 50380+adj;
		if (int_cnt>239) {
			OCR1A = T_VAL;
			int_cnt=0;
			
			calc_zeit();
		}
	}
	
	// Tastaturabfrage und impizite Entprellung
	
	if ( !(PINC & (1<<PINC1)) ) {
		key_code=1;
		key_long=0;
	}
	if ( !(PINC & (1<<PINC3)) ) {
		key_code=2;
		key_long=0;
	}
	if ( !(PINC & (1<<PINC2)) ) { // OK-Taste
		key_code=3;
		key_long++;
	}
	if (  (PINC & ((1<<PINC1)|(1<<PINC2)|(1<<PINC3)))==0x0e) {
		key_code=0;
		key_long=0;
	}
	
	// PWM fr LCD-Backlight
	bl_ist++;
	backlight(bl_mode);
}

void lcd_enable(void) // erzeugt den Enable-Puls
{
   PORTD |= (1<<PD5);
   asm("nop");
   asm("nop");
   asm("nop");
   PORTD &= ~(1<<PD5);
}

void lcd_busy_wait(void) // wartet bis lcd fertig ist
{
	uint8_t i=1;
	
	DDRD = 0xF0;
	PORTD &= ~(1<<PD4); // PD4 = RS-Leitung 0
	PORTD |=  (1<<PD6); // PD6 = R/W-Leitung 1
	PORTD |=  (1<<PD5); // enable 1
	while(i!=0) {
		i=PIND << 4; // liest die ersten 4 BIT ein
		i=(PIND & (1<<PIND3)); // Fragt PD3 auf 1 ab
	}
	PORTD &= ~(1<<PD5); // enable 0
	PORTD &= ~(1<<PD6); //R/W-Leitung 0
	DDRD = 0xFF;
}


 void lcd_send(uint8_t temp1,uint8_t c_flag) // sendet Daten ans LCD C_flag 0:Daten 1:Kommando
{
	uint8_t temp2 = temp1;
	
	uint8_t bl_status;

	temp1 = temp1 >> 4;   // oberes Nibble holen
	temp1 = temp1 & 0x0F; // maskieren
	
	bl_status=(PIND & (1<<PIND7)); // fragt status von PD7 (BL) ab
	if(c_flag==1)	PORTD =  temp1 | bl_status ;        // setzen
	else PORTD = temp1 | (1<<PD4) | bl_status ; // inkl. PD4 = RS-Leitung setzen 
	lcd_enable();
	temp2 = temp2 & 0x0F; // unteres Nibble holen und maskieren
	
	if(c_flag==1)	PORTD =  temp2 | bl_status;        // setzen
	else PORTD = temp2 | (1<<PD4) | bl_status ; // inkl. PD4 = RS-Leitung setzen
	lcd_enable();
	
	lcd_busy_wait();
}
 
void lcd_init(void) // Muss ganz am Anfang des Programms aufgerufen werden.
{
	uint8_t temp3 = 250;

	while(temp3 != 0)
	{
	_delay_ms(1);  // 1ms = 250ms warten
	temp3--;
	}
	PORTD = 0x03; // muss 3mal hintereinander gesendet werden zur Initialisierung 
	lcd_enable();
	PORTD = 0x03;
	lcd_enable();
	PORTD = 0x03;
	lcd_enable();
	PORTD = 0x02; // 4bit-Modus einstellen  
	lcd_enable();
	lcd_busy_wait();
	lcd_send(0x28,1);  // 4Bit / 2 Zeilen / 5x8        
	lcd_busy_wait(); 
	lcd_send(0x0C,1); // Display ein / Cursor aus / kein Blinken        
	lcd_busy_wait();
	lcd_send(0x04,1);  // inkrement / kein Scrollen
	lcd_busy_wait();
}


void lcd_clear(void) // lscht das display
{
   lcd_send(0x01,1);
} 

void lcd_home1(void) // Cursor auf stelle 1
{
   lcd_send(0x80,1);
} 

void lcd_home2(void) // Cursor auf stelle 1 Zeile 2
{
   lcd_send(0xC0,1);
}

void lcd_text(char array[]) // gib text auf LCD aus
{
	uint8_t i=0;
  		
   	while(array[i]!='\0') {
		lcd_send(array[i],0);
		i++;
	}
}



void itos(char string[], uint8_t zahl) // Konvertiert 2 Stellen
{
	uint8_t temp=zahl;
	uint8_t merker=0;
	
	while(temp>9) {
		temp=temp-10;
		merker++;
	}
	string[0]=merker+0x30;
	string[1]=(zahl-(merker*10))+0x30;
}

void print(datestruct time, uint8_t mode, uint8_t blink_pos, uint8_t blink_enable) // print uhr und datum - managed blinken
{
	char text[]="0123456789abcdef";
	char temp[]="12";
	char *wday_text[] = {"So","Mo","Di","Mi","Do","Fr","Sa"};
	char *menu_text[] = {"Schaltzeit A1 ","Schaltzeit B1 ","Schaltzeit A2 ","Schaltzeit B2 ","Uhr/Datum     ","Handbetrieb   ","Dimmer        ","Adjustment    ","Menu verlassen"}; //Strings mssen gleich lang sein
	
	switch(mode){
	
	case 0 : 	lcd_home1(); // Text "Eingabe:"
				strcpy(text,"Version 1.0");
				lcd_text(text);
				lcd_home2();
				strcpy(text,__DATE__);
				break;
	
	case 1 :	lcd_home1(); //Uhrzeit
				strcpy(text,"   ");
				itos(temp,time.hour);
				if((blink_pos==1)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				strcat(text, ":");
				itos(temp,time.min);
				if((blink_pos==2)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				strcat(text, ":");
				itos(temp,time.sec);
				if((blink_pos==3)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				break;
				
	case 2 :	lcd_home2(); // datum
				strcpy(text,"");
				if((blink_pos==4)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text,wday_text[time.wday]);
				strcat(text, " ");
				itos(temp,time.day);
				if((blink_pos==5)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				strcat(text, ".");
				itos(temp,time.month);
				if((blink_pos==6)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				strcat(text, ".");
				itos(temp,time.year);
				if((blink_pos==7)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				break;
				
	case 3 :	lcd_home1(); // Text "Eingabe:"
				strcpy(text,"Eingabe:");
				lcd_text(text);
				lcd_home2();
				strcpy(text,menu_text[blink_pos]);
				break;
				
	case 4 :	lcd_home2(); // Eingabe Schaltzeiten aus
				strcpy(text,"Aus: ");
				itos(temp,time.hour);
				if((blink_pos==1)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				strcat(text, ":");
				itos(temp,time.min);
				if((blink_pos==2)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				strcat(text, ":");
				itos(temp,time.sec);
				if((blink_pos==3)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				break;
	
	case 5 : 	lcd_home1(); // Eingabe Schaltzeiten ein
				strcpy(text,"Ein: ");
				itos(temp,time.hour);
				if((blink_pos==1)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				strcat(text, ":");
				itos(temp,time.min);
				if((blink_pos==2)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				strcat(text, ":");
				itos(temp,time.sec);
				if((blink_pos==3)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				break;

	case 6 :	lcd_send(0x8C,1); // Cursor auf Postion 13 in Zeile 1 
				if(kreis1==1) strcpy(text," On ");
				else strcpy(text," Off");
				break;

	case 7 :	lcd_send(0xCC,1); // Cursor auf Postion 13 in Zeile 2
				if(kreis2==1) strcpy(text," On ");
				else strcpy(text," Off");
				break;
				
	case 8:		lcd_home1();
				strcpy(text,"Kreis 1:");
				break;
				
	case 9:		lcd_home2();
				strcpy(text,"Kreis 2:");
				break;
				
	case 10 :	lcd_home1(); // gibt "Dimmer:" in Zeile 1 aus
				strcpy(text,"Dimmer:");
				lcd_text(text);
				lcd_home2(); // gibt Zahl in Zeile 2 incl. Blinken beim Einstellen aus
				strcpy(text,"       ");
				itos(temp,bl_soll);
				if((blink_pos==1)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				break;
	
	case 11 :	lcd_home1(); // gibt "Adjustment:" in Zeile 1 aus
				strcpy(text,"Adjust:");
				lcd_text(text);
				lcd_home2(); // gibt Zahl in Zeile 2 incl. Blinken beim Einstellen aus
				strcpy(text,"       ");
				itos(temp,adj);
				if((blink_pos==1)&&(blink_enable==1)) strcat(text, "--");
				else strcat(text, temp);
				break;
	}
	
	lcd_text(text);
}



void alter_time( uint8_t index, volatile datestruct *time) // stellt uhr und datum abh von tastendck
{
	// index=0 is default-Stellung, da wird nichts verstellt
	
	switch(index) {
	case 1 :	if(key_code==1)	{ if (time->hour<23) time->hour++; else time->hour=0; }
				if(key_code==2) 	{ if (time->hour>0) time->hour--; else time->hour=23; }
				break;
	
	case 2 :	if(key_code==1)	{ if (time->min<59) time->min++; else time->min=0; }
				if(key_code==2) 	{ if (time->min>0) time->min--; else time->min=59; }
				break;
				
	case 3 :   	if(key_code==1)	{ if (time->sec<59) time->sec++; else time->sec=0; }
				if(key_code==2) 	{ if (time->sec>0) time->sec--; else time->sec=59; }
				break;
				
	case 4 :	if(key_code==1) 	{ if (time->wday<6) time->wday++; else time->wday=0; }
				if(key_code==2) 	{ if (time->wday>0) time->wday--; else time->wday=6; }
				break;

	case 5 :	if(key_code==1)	{ if (time->day<31) time->day++; else time->day=0; }
				if(key_code==2)	{ if (time->day>0) time->day--; else time->day=31; }
				break;
				
	case 6 :	if(key_code==1)  	{ if (time->month<12) time->month++; else time->month=0; }
				if(key_code==2)  	{ if (time->month>0) time->month--; else time->month=12; }
				break;
				
	case 7 :	if(key_code==1)  	{ if (time->year<99) time->year++; else time->year=0; }
				if(key_code==2)  	{ if (time->year>0) time->year--; else time->year=99; }
				break;
	}
}

void schalten(void) // schaltet die 2 Keise
{	
	if(kreis1==1) {
		PORTC |= (1<<PC5); // einschalt-Relais
		PORTC |= (1<<PC4); // Umschaltrelais
	} 
	
	if(kreis2==1) {
		PORTC |= (1<<PC5); // einschalt-Relais
		PORTC &= ~(1<<PC4); // Umschaltrelais
	}
	
	if((kreis1==0) && (kreis2==0)) {
		PORTC &= ~(1<<PC5);
	}
}

void dimmen(void)  // Eingaberoutine fr Dimm-Wert in Gl. Variable bl_status
{
	uint8_t ok_pressed=0;
	uint8_t old_key=3; // Aus meune noch OK enthalten daher init 3
	uint8_t old_sec=0;
	uint8_t blink=0;
	
	print(zeit,10,1,0);
	while(ok_pressed==0)
	{
		if(old_key!=key_code) { // Wurde irgendeine Taste gedrckt ?
			if(key_code==3)  { // OK-Taste gedrckt ?
				ok_pressed=1;
			}
			if(key_code==1)  { // + -Taste gedrckt ?
				if(bl_soll<5) bl_soll++; else bl_soll=1;
				bl_ist=0;
			}
			if(key_code==2)  { // - -Taste gedrckt ?
				if(bl_soll>1) bl_soll--; else bl_soll=5;
				bl_ist=0;
			}
			old_key=key_code;
			print(zeit,10,1,0); 
			blink=0; // Stellt sicher, dass es nach Tastendruck sofort blinkt
		}
				
		if(old_sec!=zeit.sec) {
			if(blink==0) blink=1; else blink=0;
			print(zeit,10,1,blink); 
			old_sec=zeit.sec;
		}
	}
}



void adjust(void)  // Eingaberoutine fr Ajustwert
{
	uint8_t ok_pressed=0;
	uint8_t old_key=3; // Aus meune noch OK enthalten daher init 3
	uint8_t old_sec=0;
	uint8_t blink=0;
	
	print(zeit,11,1,0);
	while(ok_pressed==0)
	{
		if(old_key!=key_code) { // Wurde irgendeine Taste gedrckt ?
			if(key_code==3)  { // OK-Taste gedrckt ?
				ok_pressed=1;
			}
			if(key_code==1)  { // + -Taste gedrckt ?
				adj++;
			}
			if(key_code==2)  { // - -Taste gedrckt ?
				adj--;
			}
			old_key=key_code;
			print(zeit,11,1,0); 
			blink=0; // Stellt sicher, dass es nach Tastendruck sofort blinkt
		}
				
		if(old_sec!=zeit.sec) {
			if(blink==0) blink=1; else blink=0;
			print(zeit,11,1,blink); 
			old_sec=zeit.sec;
		}
	}
}



void handbetrieb(void)  // Eingaberoutine fr Manellen Betrieb
{
	uint8_t ok_pressed=0;
	uint8_t old_key=3;
	
	lcd_clear();
	print(zeit,8,0,0);
	print(zeit,9,0,0);
	print(zeit,6,0,0);
	print(zeit,7,0,0);
	
	while(ok_pressed==0)
	{
		if(old_key!=key_code) { // Wurde irgendeine Taste gedrckt ?
			if(key_code==3)  { // OK-Taste gedrckt ?
				ok_pressed=1;
			}
			
			if(key_code==1) {
				if(kreis1==1) kreis1=0;
				else {
					kreis1=1;
					kreis2=0;
				}
			}	
			
			
			if(key_code==2) {
				if(kreis2==1) kreis2=0;
				else {
					kreis2=1;
					kreis1=0;
				}
			}
			
			old_key=key_code;
			schalten();
			print(zeit,6,0,0);
			print(zeit,7,0,0);
		}
		
		
		
	}
}

void einstellen(volatile datestruct *time, uint8_t mode)  // Eingaberoutine fr uhrzeiten und datum
{
	uint8_t ok_pressed=0;
	uint8_t old_key=0;
	uint8_t old_sec=0;
	uint8_t set_ind=0;
	uint8_t set_ind_max=0;
	uint8_t blink=0;
	
	if(mode==1) set_ind_max=7;
	if((mode==0)||(mode==2)) set_ind_max=3;
	
	while(ok_pressed==0)
	{
		if(old_key!=key_code) { // Wurde irgendeine Taste gedrckt ?
			if(key_code==3)  { // OK-Taste gedrckt ?
				set_ind++;
				if(set_ind>set_ind_max) {
					set_ind=0;
					ok_pressed=1;
				}
			}
			alter_time(set_ind,time);
			old_key=key_code;
			
			switch (mode) {
			case 0 : 		print(*time,5,set_ind,0);
							break;
			case 1 :		
							print(*time,1,set_ind,0);
							print(*time,2,set_ind,0);
							break;
			
			case 2 :		print(*time,4,set_ind,0);
							break;
			}
			blink=0; // Stellt sicher, dass es nach Tastendruck sofort blinkt
		}
		
		if(old_sec!=zeit.sec) {
			
			if(blink==0) blink=1; else blink=0;
			switch (mode) {
			case 0 : 		print(*time,5,set_ind,blink);
							break;
			case 1 :		
							print(*time,1,set_ind,blink);
							print(*time,2,set_ind,blink);
							break;
			
			case 2 :		print(*time,4,set_ind,blink);
							break;
			}
			old_sec=zeit.sec;
		}
	}
}

void copy_zeit(volatile datestruct *tozeit, volatile datestruct *fromzeit) // Kopiert zeiten fr Komortfunktionen
{
	tozeit->hour=fromzeit->hour;
	tozeit->min=fromzeit->min;
	tozeit->sec=fromzeit->sec;
	tozeit->day=fromzeit->day;
	tozeit->wday=fromzeit->wday;
	tozeit->month=fromzeit->month;
	tozeit->year=fromzeit->year;
	tozeit->dls_flag=fromzeit->dls_flag;
	tozeit->leapyear=fromzeit->leapyear;
}



uint8_t menu(void) // 1-Level Menue
{
	uint8_t ok_pressed=0;
	uint8_t old_key=3;
	uint8_t ind=0;

	lcd_clear();
	print(zeit,3,ind,0);
	while(ok_pressed==0)
	{
		if(old_key!=key_code) {
			if(key_code==3)  { // OK-Taste gedrckt ?
				ok_pressed=1;
			}
			if(key_code==1) { if (ind<8) ind++; else ind=0; }
			if(key_code==2) { if (ind>0) ind--; else ind=8; }
			old_key=key_code;
			print(zeit,3,ind,0);
		}
	}
	lcd_clear();
	return(ind);
}

uint8_t compare(datestruct time1, datestruct time2) // vergleicht die aktuelle Zeit mit den Weckzeiten
{
	uint8_t hcmp=0;
	uint8_t mcmp=0;
	uint8_t scmp=0;
	
		
	if(time1.hour == time2.hour) hcmp=1;
		
	if(time1.min == time2.min) mcmp=1;
		
	if(time1.sec == time2.sec) scmp=1;
		
	if((hcmp==1)&&(mcmp==1)&&(scmp==1)) return(1);
	return(0);
}



int main(void)
{
	uint8_t old_sec=0;
	uint8_t auswahl=0;
	uint8_t old_key=0;
	
		
	TCCR1A = 0x00;
	TCCR1B = 1<<WGM12 ^ 1<<CS10; // set CTC-Mode "4" + no prescaler
	OCR1A = T_VAL;
	TIMSK = 1<<OCIE1A;
	
	sei(); 
	
	DDRD = 0xFF;	// Port D komplett als Ausgang einstellen
	DDRC = 0x30;	// Port C konfigureren

	lcd_init();
	
	lcd_clear();
	
	print(zeit,0,0,0); // print compile-date
	
	bl_mode=1;
	backlight(bl_mode); //schaltet backlight 
	
	
	while (1) {
		if(key_long>250) { // OK lang drcken ruft menue auf
			auswahl=menu();
			
			if(auswahl==0) { // Schaltzeit1 gewhlt
				if(szeit1_on.year==0) copy_zeit(&szeit1_on,&zeit); // vorbelegung mit akt. Zeit wenn Jahr=0 (initial) ist
				einstellen(&szeit1_on,0);
				if(szeit1_off.year==0) copy_zeit(&szeit1_off,&szeit1_on); // vorbelegung mit Startzeit wenn Jahr=0 (initial) ist
				einstellen(&szeit1_off,2);
			}
			
			if(auswahl==1) { // Schaltzeit2 gewhlt
				if(szeit2_on.year==0) copy_zeit(&szeit2_on,&zeit);
				einstellen(&szeit2_on,0);
				if(szeit2_off.year==0) copy_zeit(&szeit2_off,&szeit2_on);
				einstellen(&szeit2_off,2);
			}
			
			if(auswahl==2) { // Schaltzeit3 gewhlt
				if(szeit3_on.year==0) copy_zeit(&szeit3_on,&zeit); // vorbelegung mit akt. Zeit wenn Jahr=0 (initial) ist
				einstellen(&szeit3_on,0);
				if(szeit3_off.year==0) copy_zeit(&szeit3_off,&szeit3_on); // vorbelegung mit Startzeit wenn Jahr=0 (initial) ist
				einstellen(&szeit3_off,2);
			}
			
			if(auswahl==3) { // Schaltzeit4 gewhlt
				if(szeit4_on.year==0) copy_zeit(&szeit4_on,&zeit);
				einstellen(&szeit4_on,0);
				if(szeit4_off.year==0) copy_zeit(&szeit4_off,&szeit4_on);
				einstellen(&szeit4_off,2);
			}
			if(auswahl==4) { // Zeit+Datum einstellen
				einstellen(&zeit,1); 
			}
			if(auswahl==5) { // Handbetrieb
				handbetrieb();
			} 
			
			if(auswahl==6) { // BL-Einstellung
				dimmen();
			}
			if(auswahl==7) { // Adjust
				adjust();
			}
			if(auswahl==8) {} ; // verlassen
			
			lcd_clear();
		}
		
		// fragt tasten 1+2 (nicht ok-taste) ab
		if(old_key!=key_code) {
			if((key_code==1)||(key_code==2))  { 
				if(bl_mode==0) bl_mode=1; else bl_mode=0;
				bl_ist=bl_soll-1; // stellt sicher,dass ndeung sofort wirksam wird
				backlight(bl_mode);
				
			}
			old_key=key_code;
		}
		
		// gibt pro sekunde einmal die Uhrzeit+Datum+Kreisstatus aus und schaltet
		if(old_sec!=zeit.sec) {
		
			if(compare(zeit,szeit1_on)==1)  kreis1=1;
			if(compare(zeit,szeit1_off)==1) kreis1=0;
			if(compare(zeit,szeit2_on)==1)  kreis2=1;
			if(compare(zeit,szeit2_off)==1) kreis2=0;
			schalten();
			if(compare(zeit,szeit3_on)==1)  kreis1=1;
			if(compare(zeit,szeit3_off)==1) kreis1=0;
			if(compare(zeit,szeit4_on)==1)  kreis2=1;
			if(compare(zeit,szeit4_off)==1) kreis2=0;
			
			schalten();
			
			print(zeit,1,0,0); // Schreibt Uhrzeit
			print(zeit,2,0,0); // Schreibt Datum
			
			print(zeit,6,0,0); // Schreibt Status Schaltkreis 1
			print(zeit,7,0,0); // Schreibt Status Schaltkreis 2
			old_sec=zeit.sec;
		}
	}
	return(0);
}




